#!/bin/sh

LOCAL_GRIDMAPFILE="/etc/grid-security/grid-mapfile.local"
DENY_GRIDMAPFILE="/etc/grid-security/grid-mapfile.deny"
DENY_DB_GRIDMAPFILE="/etc/grid-security/grid-mapfile.deny.db"
DST_GRIDMAPFILE="/etc/grid-security/grid-mapfile"
STATE_DIR=${QCG_STATE_DIR:-/var/run/qcg/qcg-comp/}

TMP_GRIDMAPFILE=`mktemp ${STATE_DIR}/gridmapfile.XXXXXXXXXX`
GENERATED_GRIDMAPFILE=`mktemp ${STATE_DIR}/generated-gridmapfile.XXXXXXXXXX`

START_TIME=`date +%s`

if [ -f $LOCAL_GRIDMAPFILE ]
then 
	cat $LOCAL_GRIDMAPFILE > $TMP_GRIDMAPFILE
fi

if [ -f $DENY_DB_GRIDMAPFILE ]
then
	rm $DENY_DB_GRIDMAPFILE
fi

if [ -f $DENY_GRIDMAPFILE ]
then
	gawk ' /"\/\w+/ { if (match($0, /(".+")/, groups)); print "^plgridX509CertificateDN=",groups[1] }' $DENY_GRIDMAPFILE > $DENY_DB_GRIDMAPFILE
else
	touch $DENY_DB_GRIDMAPFILE
fi

touch ${STATE_DIR}/plggridmapfilegenerator-defaultgrants.conf
touch ${STATE_DIR}/plggridmapfilegenerator-privategrants.conf
touch ${STATE_DIR}/plggridmapfilegenerator-groups.conf
touch ${STATE_DIR}/plggridmapfilegenerator-groupgrants.conf
chmod 0600 ${STATE_DIR}/plggridmapfilegenerator-defaultgrants.conf
chmod 0600 ${STATE_DIR}/plggridmapfilegenerator-privategrants.conf
chmod 0600 ${STATE_DIR}/plggridmapfilegenerator-groups.conf
chmod 0600 ${STATE_DIR}/plggridmapfilegenerator-groupgrants.conf

#generate other configs (groups, grants, private grants, defult grants) based on orginal one
sed 's/format=.*/format=plgridDefaultGrantID, uid/g;s/plgridX509CertificateDN/plgridDefaultGrantID/g' /etc/qcg/qcg-gridmapfile/plggridmapfilegenerator.conf  | grep -v '\[filter\]' | grep -v blacklist > ${STATE_DIR}/plggridmapfilegenerator-defaultgrants.conf
sed 's/format=.*/format=plgridPrivateGrantID, uid/g;s/plgridX509CertificateDN/plgridPrivateGrantID/g' /etc/qcg/qcg-gridmapfile/plggridmapfilegenerator.conf  | grep -v '\[filter\]' | grep -v blacklist > ${STATE_DIR}/plggridmapfilegenerator-privategrants.conf
sed 's/format=.*/format=cn, memberUid/g;s/People/Group/g;s/filter=.*/filter=(memberUid=*)/g' /etc/qcg/qcg-gridmapfile/plggridmapfilegenerator.conf  | grep -v '\[filter\]' | grep -v blacklist > ${STATE_DIR}/plggridmapfilegenerator-groups.conf
sed 's/format=.*/format=cn, plgridGrantID/g;s/People/Group/g;s/filter=.*/filter=(plgridGrantID=*)/g' /etc/qcg/qcg-gridmapfile/plggridmapfilegenerator.conf  | grep -v '\[filter\]' | grep -v blacklist > ${STATE_DIR}/plggridmapfilegenerator-groupgrants.conf

#run plggridmapfile generator
echo "Generating gridmapfile"
/opt/plgrid/bin/plggridmapfilegenerator.py -c /etc/qcg/qcg-gridmapfile/plggridmapfilegenerator.conf -o $GENERATED_GRIDMAPFILE || exit 1
echo "Generating default grants file"
/opt/plgrid/bin/plggridmapfilegenerator.py -c ${STATE_DIR}/plggridmapfilegenerator-defaultgrants.conf -o ${STATE_DIR}/plgrid-default.grants.tmp || exit 1
echo "Generating private grants file"
/opt/plgrid/bin/plggridmapfilegenerator.py -c ${STATE_DIR}/plggridmapfilegenerator-privategrants.conf -o ${STATE_DIR}/plgrid-private.grants.tmp || exit 1
echo "Generating groups file"
/opt/plgrid/bin/plggridmapfilegenerator.py -c ${STATE_DIR}/plggridmapfilegenerator-groups.conf -o ${STATE_DIR}/plgrid-groups.tmp || exit 1
echo "Generating groups granst file"
/opt/plgrid/bin/plggridmapfilegenerator.py -c ${STATE_DIR}/plggridmapfilegenerator-groupgrants.conf -o ${STATE_DIR}/plgrid-group.grants.tmp || exit 1

cat $GENERATED_GRIDMAPFILE >> $TMP_GRIDMAPFILE
cp $TMP_GRIDMAPFILE $DST_GRIDMAPFILE
rm $TMP_GRIDMAPFILE
rm $GENERATED_GRIDMAPFILE
mv ${STATE_DIR}/plgrid-default.grants.tmp ${STATE_DIR}/plgrid-default.grants
mv ${STATE_DIR}/plgrid-private.grants.tmp ${STATE_DIR}/plgrid-private.grants
mv ${STATE_DIR}/plgrid-groups.tmp ${STATE_DIR}/plgrid-groups
mv ${STATE_DIR}/plgrid-group.grants.tmp ${STATE_DIR}/plgrid-group.grants

chmod 0444 $DST_GRIDMAPFILE

#try to log in for users that do not have home directories (it may trigger creating it)
awk -F \"\  '{print $2}' $DST_GRIDMAPFILE | while read USERNAME
do
	HOME_DIR=`getent passwd $USERNAME | awk -F : '{print $6}'`
	if [ ! -d "$HOME_DIR" ]
	then
		echo "Initializing environment for user $USERNAME "
		su - $USERNAME -c 'true' > /dev/null
	fi
done

END_TIME=`date +%s`
echo "Grid mapfile generator finished on "`date`" number of entries in gridmapfile: "`wc -l $DST_GRIDMAPFILE`" Processing took:" $((END_TIME - START_TIME)) " seconds."

 
